/**
* @overview JsonStringifyOnce
* @version 1.0.0
* @license GPL2
* @module JsonStringifyOnce
* @access public
*/
module.exports = function(obj, replacer, indent) {

    var printedObjects = [];
    var printedObjectKeys = [];

    /**
    * @function JsonStringifyOnce
    * @description Call the native function JSON.stringify
    * using the replacer function printOnceReplacer
    *
    * @param {string} obj - JSON object
    * @param {Function} replacer - Replacer function
    * @param {string} indent - Indent string
    *
    * @return {string} JSON.stringify
    */
    function JsonStringifyOnce(obj, replacer, indent) {
      return JSON.stringify(obj, printOnceReplacer, indent);  
    }

    /**
    * @function printOnceReplacer
    * @description Replacer function used in JsonStringifyOnce
    *
    *
    */
    function printOnceReplacer(key, value) { 

      if (printedObjects.length > 1000 * 1000) {
        return 'object too long';
      }

      var printedObjIndex = false;

      printedObjects.forEach( function(obj, index) {
      
        if (obj===value) {
          printedObjIndex = index;
        }
      
      });

      if (key == '') {
      
        //root element
        printedObjects.push(obj);
        printedObjectKeys.push("root");
        return value;
      
      }
      else if(printedObjIndex + "" != "false" && typeof(value) == "object")
      {
        if (printedObjectKeys[printedObjIndex] == "root") {
        
          return "(pointer to root)";
        
        }
        else
        {
        
          return "(see " + 
            ((!!value && !!value.constructor) ? 
            value.constructor.name.toLowerCase() : 
            typeof(value)) + " with key " + 
            printedObjectKeys[printedObjIndex] + ")";
        
        }
      }
      else
      {
        
        var qualifiedKey = key || "(empty key)";
        
        printedObjects.push(value);
        
        printedObjectKeys.push(qualifiedKey);

        if(replacer)
        {
          return replacer(key, value);
        }else{
          return value;
        }

      }
    }

    return JsonStringifyOnce(obj, printOnceReplacer, indent);  

};